/**
 * eobjects.org MetaModel
 * Copyright (C) 2010 eobjects.org
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */

package org.eobjects.metamodel.detect;

import java.util.Arrays;
import java.util.Map;

import org.eobjects.metamodel.data.AbstractDataSet;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.DefaultRow;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.schema.Column;

public final class ColumnTypeTransformerDataSet extends AbstractDataSet {

	private final DataSet _dataSet;
	private final Map<String, ColumnTypeTransformer> _columnTypeTransformers;
	private final SelectItem[] _selectItems;

	public ColumnTypeTransformerDataSet(DataSet dataSet,
			Map<String, ColumnTypeTransformer> columnTypeTransformers) {
		_dataSet = dataSet;
		_selectItems = dataSet.getSelectItems();
		_columnTypeTransformers = columnTypeTransformers;
	}

	public void close() {
		_dataSet.close();
	}

	@Override
	public Row getRow() {
		Object[] transformedValues = new Object[_selectItems.length];
		Row row = _dataSet.getRow();
		for (int i = 0; i < _selectItems.length; i++) {
			SelectItem selectItem = _selectItems[i];
			Column column = selectItem.getColumn();
			ColumnTypeTransformer columnTypeTransformer = _columnTypeTransformers
					.get(column.getQualifiedLabel());
			Object untransformedValue = row.getValue(i);
			if (columnTypeTransformer != null
					&& untransformedValue instanceof String) {
				transformedValues[i] = columnTypeTransformer
						.narrowValue((String) untransformedValue);
			} else {
				transformedValues[i] = untransformedValue;
			}
		}
		return new DefaultRow(_selectItems, transformedValues);
	}

	@Override
	public SelectItem[] getSelectItems() {
		return Arrays.copyOf(_selectItems, _selectItems.length);
	}

	@Override
	public boolean next() {
		if (_dataSet != null) {
			return _dataSet.next();
		}
		return false;
	}

}